﻿using System;
using System.Collections;
using System.Reflection;
using System.Collections.Generic;
namespace Persistence {

  public interface ICustomSerializer {
    bool CanSerialize(Type type);
    IEnumerable Serialize(object o);
    object DeSerialize(IEnumerable o, Type t);
  }

  public class EnumerableSerializer : ICustomSerializer {
    public bool CanSerialize(Type type) {
      if (type.GetInterface("IEnumerable") == null)
        return false;
      if (type.GetMethod("GetEnumerator", new Type[] { }) == null)
        return false;
      MethodInfo addMethod = type.GetMethod("Add");
      if (addMethod == null)
        return false;
      if (addMethod.GetParameters().Length != 1)
        return false;
      return true;
    }
    public IEnumerable Serialize(object o) {
      return (IEnumerable)o;
    }
    public object DeSerialize(IEnumerable objects, Type t) {
      object instance = Activator.CreateInstance(t);
      foreach (object o in objects) {
        t.GetMethod("Add").Invoke(instance, new object[] { o });
      }
      return instance;
    }
  }

  public class ArraySerializer : ICustomSerializer {

    public bool CanSerialize(Type type) {
      return type.IsArray;
    }

    public IEnumerable Serialize(object array) {
      foreach (object o in (Array)array) {
        yield return o;
      }
    }

    public object DeSerialize(IEnumerable elements, Type t) {
      List<object> allElements = new List<object>();
      foreach (object obj in elements) {
        allElements.Add(obj);
      }
      Array array =
        Array.CreateInstance(t.GetElementType(), allElements.Count);
      for (int i = 0; i < array.Length; i++) {
        array.SetValue(allElements[i], i);
      }
      return array;
    }

  }


}
